# -*- coding: utf-8 -*-
# vim: set noai syntax=python ts=4 sw=4:
#
# Copyright (c) 2018-2021 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
"""Wait Wait Don't Tell Me! Stats Guest Appearance Retrieval Functions
"""
from functools import lru_cache
from typing import Any, Dict, Optional

from mysql.connector import connect
from wwdtm.guest.utility import GuestUtility
from wwdtm.validation import valid_int_id


class GuestAppearances:
    """This class contains functions that retrieve guest appearance
    information from a copy of the Wait Wait Stats database.

    :param connect_dict: Dictionary containing database connection
        settings as required by mysql.connector.connect
    :param database_connection: mysql.connector.connect database
        connection
    """

    def __init__(self,
                 connect_dict: Optional[Dict[str, Any]] = None,
                 database_connection: Optional[connect] = None):
        """Class initialization method.
        """
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

        self.utility = GuestUtility(database_connection=self.database_connection)

    @lru_cache(typed=True)
    def retrieve_appearances_by_id(self, guest_id: int) -> Dict[str, Any]:
        """Returns a list of dictionary objects containing appearance
        information for the requested guest ID.

        :param guest_id: Guest ID
        :return: Dictionary containing appearance counts and list of
            appearances for a guest. If guest appearances could not be
            retrieved, an empty dictionary is returned.
        """
        if not valid_int_id(guest_id):
            return {}

        cursor = self.database_connection.cursor(named_tuple=True)
        query = ("SELECT ( "
                 "SELECT COUNT(gm.showid) FROM ww_showguestmap gm "
                 "JOIN ww_shows s ON s.showid = gm.showid "
                 "WHERE s.bestof = 0 AND s.repeatshowid IS NULL AND "
                 "gm.guestid = %s ) AS regular_shows, ( "
                 "SELECT COUNT(gm.showid) FROM ww_showguestmap gm "
                 "JOIN ww_shows s ON s.showid = gm.showid "
                 "WHERE gm.guestid = %s ) AS all_shows;")
        cursor.execute(query, (guest_id, guest_id, ))
        result = cursor.fetchone()

        if result:
            appearance_counts = {
                "regular_shows": result.regular_shows,
                "all_shows": result.all_shows,
            }
        else:
            appearance_counts = {
                "regular_shows": 0,
                "all_shows": 0,
            }

        cursor = self.database_connection.cursor(named_tuple=True)
        query = ("SELECT gm.showid AS show_id, s.showdate AS date, "
                 "s.bestof AS best_of, s.repeatshowid AS repeat_show_id, "
                 "gm.guestscore AS score, gm.exception AS score_exception "
                 "FROM ww_showguestmap gm "
                 "JOIN ww_guests g ON g.guestid = gm.guestid "
                 "JOIN ww_shows s ON s.showid = gm.showid "
                 "WHERE gm.guestid = %s "
                 "ORDER BY s.showdate ASC;")
        cursor.execute(query, (guest_id, ))
        results = cursor.fetchall()
        cursor.close()

        if results:
            appearances = []
            for appearance in results:
                info = {
                    "show_id": appearance.show_id,
                    "date": appearance.date.isoformat(),
                    "best_of": bool(appearance.best_of),
                    "repeat_show": bool(appearance.repeat_show_id),
                    "score": appearance.score if appearance.score else None,
                    "score_exception": bool(appearance.score_exception),
                }
                appearances.append(info)

            return {
                "count": appearance_counts,
                "shows": appearances,
            }
        else:
            return {
                "count": appearance_counts,
                "shows": [],
            }

    @lru_cache(typed=True)
    def retrieve_appearances_by_slug(self, guest_slug: str) -> Dict[str, Any]:
        """Returns a list of dictionary objects containing appearance
        information for the requested guest slug string.

        :param guest_slug: Guest slug string
        :return: Dictionary containing appearance counts and list of
            appearances for a guest. If guest appearances could not be
            retrieved, empty dictionary is returned.
        """
        id_ = self.utility.convert_slug_to_id(guest_slug)
        if not id_:
            return {}

        return self.retrieve_appearances_by_id(id_)



# -*- coding: utf-8 -*-
# vim: set noai syntax=python ts=4 sw=4:
#
# Copyright (c) 2018-2021 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
"""Wait Wait Don't Tell Me! Stats Show Data Utility Functions
"""
import datetime
from functools import lru_cache
from typing import Any, Dict, Optional

from mysql.connector import connect
from wwdtm.validation import valid_int_id


class ShowUtility:
    """This class contains supporting functions used to check whether
    a show ID or show date exists or to convert between show ID, and
    show dates.

    :param connect_dict: Dictionary containing database connection
        settings as required by mysql.connector.connect
    :param database_connection: mysql.connector.connect database
        connection
    """

    def __init__(self,
                 connect_dict: Optional[Dict[str, Any]] = None,
                 database_connection: Optional[connect] = None):
        """Class initialization method.
        """
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

    @lru_cache(typed=True)
    def convert_date_to_id(self,
                           year: int,
                           month: int,
                           day: int) -> Optional[int]:
        """Converts a show date to the matching show ID value.

        :param year: Year portion of a show date
        :param month: Month portion of a show date
        :param day: Day portion of a show date
        :return: Show ID, if a match is found
        """
        try:
            show_date = datetime.datetime(year, month, day)
        except ValueError:
            return None

        cursor = self.database_connection.cursor(dictionary=False)
        query = ("SELECT showid FROM ww_shows "
                 "WHERE showdate = %s "
                 "LIMIT 1;")
        cursor.execute(query, (show_date.isoformat(), ))
        result = cursor.fetchone()
        cursor.close()

        if result:
            return result[0]

        return None

    @lru_cache(typed=True)
    def convert_id_to_date(self, show_id: int) -> Optional[str]:
        """Converts a show's ID to the matching show date.

        :param show_id: Show ID
        :return: Show date, if a match is found
        """
        if not valid_int_id(show_id):
            return None

        cursor = self.database_connection.cursor(dictionary=False)
        query = ("SELECT showdate FROM ww_shows "
                 "WHERE showid = %s "
                 "LIMIT 1;")
        cursor.execute(query, (show_id, ))
        result = cursor.fetchone()
        cursor.close()

        if result:
            return result[0].isoformat()

        return None

    @lru_cache(typed=True)
    def date_exists(self,
                    year: int,
                    month: int,
                    day: int) -> bool:
        """Checks to see if a show date exists.

        :param year: Year portion of a show date
        :param month: Month portion of a show date
        :param day: Day portion of a show date
        :return: True or False, based on whether the show date exists
        """
        try:
            show_date = datetime.datetime(year, month, day)
        except ValueError:
            return False

        cursor = self.database_connection.cursor(dictionary=False)
        query = ("SELECT showid FROM ww_shows "
                 "WHERE showdate = %s "
                 "LIMIT 1;")
        cursor.execute(query, (show_date.isoformat(), ))
        result = cursor.fetchone()
        cursor.close()

        return bool(result)

    @lru_cache(typed=True)
    def id_exists(self, show_id: int) -> bool:
        """Checks to see if a show ID exists.

        :param show_id: Show ID
        :return: True or False, based on whether the show ID exists
        """
        if not valid_int_id(show_id):
            return False

        cursor = self.database_connection.cursor(dictionary=False)
        query = ("SELECT showid FROM ww_shows "
                 "WHERE showid = %s "
                 "LIMIT 1;")
        cursor.execute(query, (show_id, ))
        result = cursor.fetchone()
        cursor.close()

        return bool(result)



# -*- coding: utf-8 -*-
# vim: set noai syntax=python ts=4 sw=4:
#
# Copyright (c) 2018-2021 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
"""Explicitly listing all modules in this package"""

from wwdtm import validation

from wwdtm.guest import (Guest,
                         GuestAppearances,
                         GuestUtility)
from wwdtm.host import (Host,
                        HostAppearances,
                        HostUtility)
from wwdtm.location import (Location,
                            LocationRecordings,
                            LocationUtility)
from wwdtm.panelist import (Panelist,
                            PanelistAppearances,
                            PanelistScores,
                            PanelistStatistics,
                            PanelistUtility)
from wwdtm.scorekeeper import (Scorekeeper,
                               ScorekeeperAppearances,
                               ScorekeeperUtility)
from wwdtm.show import (Show,
                        ShowInfo,
                        ShowInfoMultiple,
                        ShowUtility)


VERSION = "2.0.0-rc.3"



# -*- coding: utf-8 -*-
# vim: set noai syntax=python ts=4 sw=4:
#
# Copyright (c) 2018-2021 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
"""Wait Wait Don't Tell Me! Stats Supplemental Show Information
Retrieval Functions for Multiple Shows
"""
from functools import lru_cache
from typing import Any, Dict, List, Optional

from mysql.connector import connect
from slugify import slugify
from wwdtm.show.utility import ShowUtility
from wwdtm.location.location import LocationUtility
from wwdtm.validation import valid_int_id


class ShowInfoMultiple:
    """This class contains functions that retrieve show supplemental
    show information, including panelist, guest and Bluff the Listener
    information for multiple shows from a copy of the Wait Wait Stats
    database

    :param connect_dict: Dictionary containing database connection
        settings as required by mysql.connector.connect
    :param database_connection: mysql.connector.connect database
        connection
    """

    def __init__(self,
                 connect_dict: Optional[Dict[str, Any]] = None,
                 database_connection: Optional[connect] = None):
        """Class initialization method.
        """
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

        self.utility = ShowUtility(database_connection=self.database_connection)
        self.loc_util = LocationUtility(database_connection=self.database_connection)

    def retrieve_bluff_info_all(self) -> Dict[int, List[Dict[str, Any]]]:
        """Returns a dictionary containing Bluff the Listener information
        for all shows.

        :return: Dictionary containing correct and chosen Bluff the
            Listener information.
        """

        bluff_info = {}

        cursor = self.database_connection.cursor(named_tuple=True)
        query = ("SELECT s.showid AS show_id, "
                 "blm.chosenbluffpnlid AS panelist_id, "
                 "p.panelist AS name, p.panelistslug AS slug "
                 "FROM ww_showbluffmap blm "
                 "JOIN ww_shows s ON s.showid = blm.showid "
                 "JOIN ww_panelists p ON "
                 "p.panelistid = blm.chosenbluffpnlid;")
        cursor.execute(query)
        chosen_results = cursor.fetchall()

        query = ("SELECT s.showid AS show_id, "
                 "blm.correctbluffpnlid AS panelist_id, "
                 "p.panelist AS name, p.panelistslug AS slug "
                 "FROM ww_showbluffmap blm "
                 "JOIN ww_shows s ON s.showid = blm.showid "
                 "JOIN ww_panelists p ON "
                 "p.panelistid = blm.correctbluffpnlid;")
        cursor.execute(query)
        correct_results = cursor.fetchall()
        cursor.close()

        if not chosen_results or not correct_results:
            return {}

        for show in chosen_results:
            if show.show_id not in bluff_info:
                bluff_info[show.show_id] = {}

            bluff_info[show.show_id]["chosen_panelist"] = {
                "id": show.panelist_id,
                "name": show.name,
                "slug": show.slug if show.slug else slugify(show.name),
            }

        for show in correct_results:
            if show.show_id not in bluff_info:
                bluff_info[show.show_id] = {}

            bluff_info[show.show_id]["correct_panelist"] = {
                "id": show.panelist_id,
                "name": show.name,
                "slug": show.slug if show.slug else slugify(show.name),
            }

        return bluff_info

    def retrieve_bluff_info_by_ids(self, show_ids: List[int]) -> Dict[int, List[Dict[str, Any]]]:
        """Returns a dictionary containing Bluff the Listener information
        for the requested show IDs.

        :param show_ids: List of show IDs
        :return: Dictionary containing correct and chosen Bluff the
            Listener information.
        """
        for show_id in show_ids:
            if not valid_int_id(show_id):
                return {}

        bluff_info = {}

        cursor = self.database_connection.cursor(named_tuple=True)
        query = ("SELECT s.showid AS show_id, "
                 "blm.chosenbluffpnlid AS panelist_id, "
                 "p.panelist AS name, p.panelistslug AS slug "
                 "FROM ww_showbluffmap blm "
                 "JOIN ww_shows s ON s.showid = blm.showid "
                 "JOIN ww_panelists p ON "
                 "p.panelistid = blm.chosenbluffpnlid "
                 "WHERE s.showid IN ({ids});".format(ids=", ".join(str(v) for v in show_ids)))
        cursor.execute(query)
        chosen_results = cursor.fetchall()

        query = ("SELECT s.showid AS show_id, "
                 "blm.correctbluffpnlid AS panelist_id, "
                 "p.panelist AS name, p.panelistslug AS slug "
                 "FROM ww_showbluffmap blm "
                 "JOIN ww_shows s ON s.showid = blm.showid "
                 "JOIN ww_panelists p ON "
                 "p.panelistid = blm.correctbluffpnlid "
                 "WHERE s.showid IN ({ids});".format(ids=", ".join(str(v) for v in show_ids)))
        cursor.execute(query)
        correct_results = cursor.fetchall()
        cursor.close()

        if not chosen_results or not correct_results:
            return {}

        for show in chosen_results:
            if show.show_id not in bluff_info:
                bluff_info[show.show_id] = {}

            bluff_info[show.show_id]["chosen_panelist"] = {
                "id": show.panelist_id,
                "name": show.name,
                "slug": show.slug if show.slug else slugify(show.name),
            }

        for show in correct_results:
            if show.show_id not in bluff_info:
                bluff_info[show.show_id] = {}

            bluff_info[show.show_id]["correct_panelist"] = {
                "id": show.panelist_id,
                "name": show.name,
                "slug": show.slug if show.slug else slugify(show.name),
            }

        return bluff_info

    def retrieve_core_info_all(self) -> Dict[int, Dict[str, Any]]:
        """Returns a dictionary with core information for all shows.

        :return: Dictionary containing host, scorekeeper, location,
            description and notes. If show core information could not be
            retrieved, an empty dictionary will be returned.
        """

        cursor = self.database_connection.cursor(named_tuple=True)
        query = ("SELECT s.showid AS show_id, s.showdate AS date, "
                 "s.bestof AS best_of, s.repeatshowid AS repeat_show_id, "
                 "l.locationid AS location_id, l.city, l.state, "
                 "l.venue, l.locationslug AS location_slug, h.hostid AS host_id, "
                 "h.host, h.hostslug AS host_slug, hm.guest as host_guest, "
                 "sk.scorekeeperid AS scorekeeper_id, sk.scorekeeper, "
                 "sk.scorekeeperslug AS scorekeeper_slug, "
                 "skm.guest AS scorekeeper_guest, "
                 "skm.description AS scorekeeper_description, "
                 "sd.showdescription AS show_description, "
                 "sn.shownotes AS show_notes "
                 "FROM ww_shows s "
                 "JOIN ww_showlocationmap lm ON lm.showid = s.showid "
                 "JOIN ww_locations l ON l.locationid = lm.locationid "
                 "JOIN ww_showhostmap hm ON hm.showid = s.showid "
                 "JOIN ww_hosts h ON h.hostid = hm.hostid "
                 "JOIN ww_showskmap skm ON skm.showid = s.showid "
                 "JOIN ww_scorekeepers sk ON "
                 "sk.scorekeeperid = skm.scorekeeperid "
                 "JOIN ww_showdescriptions sd ON sd.showid = s.showid "
                 "JOIN ww_shownotes sn ON sn.showid = s.showid "
                 "ORDER BY s.showdate ASC;")
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return {}

        shows = {}
        for show in results:
            location_info = {
                "id": show.location_id,
                "slug": show.location_slug,
                "city": show.city,
                "state": show.state,
                "venue": show.venue,
            }

            if not show.location_slug:
                location_info["slug"] = self.loc_util.slugify_location(location_id=show.location_id,
                                                                       venue=show.venue,
                                                                       city=show.city,
                                                                       state=show.state)

            host_info = {
                "id": show.host_id,
                "name": show.host,
                "slug": show.host_slug if show.host_slug else slugify(show.host),
                "guest": bool(show.host_guest),
            }

            scorekeeper_info = {
                "id": show.scorekeeper_id,
                "name": show.scorekeeper,
                "slug": show.scorekeeper_slug if show.scorekeeper_slug else slugify(show.scorekeeper),
                "guest": bool(show.scorekeeper_guest),
                "description": show.scorekeeper_description if show.scorekeeper_description else None,
            }

            if show.show_description:
                description = str(show.show_description).strip()
            else:
                description = None

            if show.show_notes:
                notes = str(show.show_notes).strip()
            else:
                notes = None

            show_info = {
                "id": show.show_id,
                "date": show.date.isoformat(),
                "best_of": bool(show.best_of),
                "repeat_show": bool(show.repeat_show_id),
                "original_show_id": None,
                "original_show_date": None,
                "description": description,
                "notes": notes,
                "location": location_info,
                "host": host_info,
                "scorekeeper": scorekeeper_info,
            }

            repeat_show_id = show.repeat_show_id
            if repeat_show_id:
                original_date = self.utility.convert_id_to_date(repeat_show_id)
                show_info["original_show_id"] = repeat_show_id
                show_info["original_show_date"] = original_date
            else:
                show_info.pop("original_show_id", None)
                show_info.pop("original_show_date", None)

            shows[show.show_id] = show_info

        return(shows)

    def retrieve_core_info_by_ids(self, show_ids: List[int]) -> Dict[int, Dict[str, Any]]:
        """Returns a dictionary with core information for the requested
        show IDs.

        :param show_ids: List of show IDs
        :return: Dictionary containing host, scorekeeper, location,
            description and notes. If show core information could not be
            retrieved, an empty dictionary will be returned.
        """
        for show_id in show_ids:
            if not valid_int_id(show_id):
                return {}

        cursor = self.database_connection.cursor(named_tuple=True)
        query = ("SELECT s.showid AS show_id, s.showdate AS date, "
                 "s.bestof AS best_of, s.repeatshowid AS repeat_show_id, "
                 "l.locationid AS location_id, l.city, l.state, "
                 "l.venue, l.locationslug AS location_slug, h.hostid AS host_id, "
                 "h.host, h.hostslug AS host_slug, hm.guest as host_guest, "
                 "sk.scorekeeperid AS scorekeeper_id, sk.scorekeeper, "
                 "sk.scorekeeperslug AS scorekeeper_slug, "
                 "skm.guest AS scorekeeper_guest, "
                 "skm.description AS scorekeeper_description, "
                 "sd.showdescription AS show_description, "
                 "sn.shownotes AS show_notes "
                 "FROM ww_shows s "
                 "JOIN ww_showlocationmap lm ON lm.showid = s.showid "
                 "JOIN ww_locations l ON l.locationid = lm.locationid "
                 "JOIN ww_showhostmap hm ON hm.showid = s.showid "
                 "JOIN ww_hosts h ON h.hostid = hm.hostid "
                 "JOIN ww_showskmap skm ON skm.showid = s.showid "
                 "JOIN ww_scorekeepers sk ON "
                 "sk.scorekeeperid = skm.scorekeeperid "
                 "JOIN ww_showdescriptions sd ON sd.showid = s.showid "
                 "JOIN ww_shownotes sn ON sn.showid = s.showid "
                 "WHERE s.showid IN ({ids}) "
                 "ORDER BY s.showdate ASC;".format(ids=", ".join(str(v) for v in show_ids)))
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return {}

        shows = {}
        for show in results:
            location_info = {
                "id": show.location_id,
                "slug": show.location_slug,
                "city": show.city,
                "state": show.state,
                "venue": show.venue,
            }

            if not show.location_slug:
                location_info["slug"] = self.loc_util.slugify_location(location_id=show.location_id,
                                                                       venue=show.venue,
                                                                       city=show.city,
                                                                       state=show.state)

            host_info = {
                "id": show.host_id,
                "name": show.host,
                "slug": show.host_slug if show.host_slug else slugify(show.host),
                "guest": bool(show.host_guest),
            }

            scorekeeper_info = {
                "id": show.scorekeeper_id,
                "name": show.scorekeeper,
                "slug": show.scorekeeper_slug if show.scorekeeper_slug else slugify(show.scorekeeper),
                "guest": bool(show.scorekeeper_guest),
                "description": show.scorekeeper_description if show.scorekeeper_description else None,
            }

            if show.show_description:
                description = str(show.show_description).strip()
            else:
                description = None

            if show.show_notes:
                notes = str(show.show_notes).strip()
            else:
                notes = None

            show_info = {
                "id": show.show_id,
                "date": show.date.isoformat(),
                "best_of": bool(show.best_of),
                "repeat_show": bool(show.repeat_show_id),
                "original_show_id": None,
                "original_show_date": None,
                "description": description,
                "notes": notes,
                "location": location_info,
                "host": host_info,
                "scorekeeper": scorekeeper_info,
            }

            repeat_show_id = show.repeat_show_id
            if repeat_show_id:
                original_date = self.utility.convert_id_to_date(repeat_show_id)
                show_info["original_show_id"] = repeat_show_id
                show_info["original_show_date"] = original_date
            else:
                show_info.pop("original_show_id", None)
                show_info.pop("original_show_date", None)

            shows[show.show_id] = show_info

        return(shows)

    def retrieve_guest_info_all(self) -> Dict[int, List[Dict[str, Any]]]:
        """Returns a list of dictionary objects containing Not My Job
        guest information for all shows.

        :return: Dictionary containing Not My Job guest information. If
            Not My Job information could not be retrieved, an empty list
            will be returned.
        """
        cursor = self.database_connection.cursor(named_tuple=True)
        query = ("SELECT s.showid AS show_id, gm.guestid AS guest_id, "
                 "g.guest AS name, g.guestslug AS slug, "
                 "gm.guestscore AS score, gm.exception AS score_exception "
                 "FROM ww_showguestmap gm "
                 "JOIN ww_guests g ON g.guestid = gm.guestid "
                 "JOIN ww_shows s ON s.showid = gm.showid "
                 "ORDER BY s.showdate ASC, gm.showguestmapid ASC;")
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        shows = {}
        for guest in results:
            if guest.show_id not in shows:
                shows[guest.show_id] = []

            shows[guest.show_id].append({
                "id": guest.guest_id,
                "name": guest.name,
                "slug": guest.slug if guest.slug else slugify(guest.name),
                "score": guest.score if guest.score else None,
                "score_exception": bool(guest.score_exception),
            })

        return shows

    def retrieve_guest_info_by_ids(self, show_ids: List[int]) -> Dict[int, List[Dict[str, Any]]]:
        """Returns a list of dictionary objects containing Not My Job
        guest information for the requested show IDs.

        :param show_ids: List of show IDs
        :return: Dictionary containing Not My Job guest information. If
            Not My Job information could not be retrieved, an empty list
            will be returned.
        """
        for show_id in show_ids:
            if not valid_int_id(show_id):
                return {}

        cursor = self.database_connection.cursor(named_tuple=True)
        query = ("SELECT s.showid AS show_id, gm.guestid AS guest_id, "
                 "g.guest AS name, g.guestslug AS slug, "
                 "gm.guestscore AS score, gm.exception AS score_exception "
                 "FROM ww_showguestmap gm "
                 "JOIN ww_guests g ON g.guestid = gm.guestid "
                 "JOIN ww_shows s ON s.showid = gm.showid "
                 "WHERE gm.showid IN ({ids}) "
                 "ORDER BY s.showdate ASC, "
                 "gm.showguestmapid ASC;".format(ids=", ".join(str(v) for v in show_ids)))
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        shows = {}
        for guest in results:
            if guest.show_id not in shows:
                shows[guest.show_id] = []

            shows[guest.show_id].append({
                "id": guest.guest_id,
                "name": guest.name,
                "slug": guest.slug if guest.slug else slugify(guest.name),
                "score": guest.score if guest.score else None,
                "score_exception": bool(guest.score_exception),
            })

        return shows

    def retrieve_panelist_info_all(self) -> Dict[int, List[Dict[str, Any]]]:
        """Returns a list of dictionary objects containing panelist
        information for all shows.

        :return: List of panelists with corresponding scores and
            ranking information. If panelist information could not be
            retrieved, an empty list will be returned.
        """

        cursor = self.database_connection.cursor(named_tuple=True)
        query = ("SELECT s.showid AS show_id, pm.panelistid AS panelist_id, "
                 "p.panelist AS name, p.panelistslug AS slug, "
                 "pm.panelistlrndstart AS start, "
                 "pm.panelistlrndcorrect AS correct, "
                 "pm.panelistscore AS score, pm.showpnlrank AS pnl_rank "
                 "FROM ww_showpnlmap pm "
                 "JOIN ww_panelists p ON p.panelistid = pm.panelistid "
                 "JOIN ww_shows s ON s.showid = pm.showid "
                 "ORDER by s.showdate ASC, pm.panelistscore DESC, "
                 "pm.showpnlmapid ASC;")
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return {}

        panelists = {}
        for panelist in results:
            if panelist.show_id not in panelists:
                panelists[panelist.show_id] = []

            panelists[panelist.show_id].append({
                "id": panelist.panelist_id,
                "name": panelist.name,
                "slug": panelist.slug if panelist.slug else slugify(panelist.name),
                "lightning_round_start": panelist.start,
                "lightning_round_correct": panelist.correct,
                "score": panelist.score if panelist.score else None,
                "rank": panelist.pnl_rank if panelist.pnl_rank else None,
            })

        return panelists

    def retrieve_panelist_info_by_ids(self, show_ids: List[int]) -> Dict[int, List[Dict[str, Any]]]:
        """Returns a list of dictionary objects containing panelist
        information for the requested show IDs.

        :param show_ids: List of show IDs
        :return: List of panelists with corresponding scores and
            ranking information. If panelist information could not be
            retrieved, an empty list will be returned.
        """
        for show_id in show_ids:
            if not valid_int_id(show_id):
                return {}

        cursor = self.database_connection.cursor(named_tuple=True)
        query = ("SELECT s.showid AS show_id, pm.panelistid AS panelist_id, "
                 "p.panelist AS name, p.panelistslug AS slug, "
                 "pm.panelistlrndstart AS start, "
                 "pm.panelistlrndcorrect AS correct, "
                 "pm.panelistscore AS score, pm.showpnlrank AS pnl_rank "
                 "FROM ww_showpnlmap pm "
                 "JOIN ww_panelists p ON p.panelistid = pm.panelistid "
                 "JOIN ww_shows s ON s.showid = pm.showid "
                 "WHERE pm.showid IN ({ids}) "
                 "ORDER by s.showdate ASC, pm.panelistscore DESC, "
                 "pm.showpnlmapid ASC;".format(ids=", ".join(str(v) for v in show_ids)))
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return {}

        panelists = {}
        for panelist in results:
            if panelist.show_id not in panelists:
                panelists[panelist.show_id] = []

            panelists[panelist.show_id].append({
                "id": panelist.panelist_id,
                "name": panelist.name,
                "slug": panelist.slug if panelist.slug else slugify(panelist.name),
                "lightning_round_start": panelist.start if panelist.start else None,
                "lightning_round_correct": panelist.correct if panelist.correct else None,
                "score": panelist.score if panelist.score else None,
                "rank": panelist.pnl_rank if panelist.pnl_rank else None,
            })

        return panelists



# -*- coding: utf-8 -*-
# vim: set noai syntax=python ts=4 sw=4:
#
# Copyright (c) 2018-2021 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
"""Testing for object :py:class:`wwdtm.show.ShowInfo`
"""
import json
from typing import Any, Dict, List

import pytest
from wwdtm.show import ShowInfoMultiple


@pytest.mark.skip
def get_connect_dict() -> Dict[str, Any]:
    """Read in database connection settings and return values as a
    dictionary.
    """
    with open("config.json", "r") as config_file:
        config_dict = json.load(config_file)
        if "database" in config_dict:
            return config_dict["database"]


@pytest.mark.parametrize("show_id", [1083])
def test_show_info_retrieve_bluff_info_all(show_id: int):
    """Testing for :py:meth:`wwdtm.show.ShowInfoMultiple.retrieve_bluff_info_all`

    :param show_id: Show ID to test retrieving show Bluff the Listener
        information from all shows retrieved
    """
    info = ShowInfoMultiple(connect_dict=get_connect_dict())
    bluffs = info.retrieve_bluff_info_all()

    assert bluffs, ("Bluff the Listener information for all shows could not "
                    "be retrieved")
    assert show_id in bluffs, ("Bluff the Listener information was not "
                               f"returned for show ID {show_id}")

    bluff = bluffs[show_id]
    assert "chosen_panelist" in bluff, ("'chosen_panelist' was not returned with "
                                        f"panelist information for show ID {show_id}")
    assert "correct_panelist" in bluff, ("'correct_panelist' was not returned with "
                                         f"panelist information for show ID {show_id}")


@pytest.mark.parametrize("show_ids", [[1083, 1162]])
def test_show_info_retrieve_bluff_info_by_ids(show_ids: List[int]):
    """Testing for :py:meth:`wwdtm.show.ShowInfoMultiple.retrieve_bluff_info_by_ids`

    :param show_ids: List of show IDs to test retrieving show Bluff the
        Listener information
    """
    info = ShowInfoMultiple(connect_dict=get_connect_dict())
    bluffs = info.retrieve_bluff_info_by_ids(show_ids)

    assert bluffs, (f"Bluff the Listener information for the show IDs {show_ids} "
                    f"could not be retrieved")

    for show_id in show_ids:
        assert show_id in bluffs, ("Bluff the Listener information was not "
                                   f"returned for show ID {show_id}")
        assert "chosen_panelist" in bluffs[show_id], ("'chosen_panelist' was not "
                                                      "returned with panelist "
                                                      "information for show ID "
                                                      f"{show_id}")
        assert "correct_panelist" in bluffs[show_id], ("'correct_panelist' was not "
                                                       "returned with panelist "
                                                       "information for show ID "
                                                       f"{show_id}")


@pytest.mark.parametrize("show_id", [1162])
def test_show_info_retrieve_core_info_all(show_id: int):
    """Testing for :py:meth:`wwdtm.show.ShowInfoMultiple.retrieve_core_info_all`

    :param show_id: Show ID to test retrieving show core information
        from all shows retrieved
    """
    info = ShowInfoMultiple(connect_dict=get_connect_dict())
    shows = info.retrieve_core_info_all()

    assert shows, "Core information for all shows could not be retrieved"
    assert show_id in shows, (f"Core information for show ID {show_id} "
                              f"could not be retrieved")

    show = shows[show_id]
    assert "id" in show, ("'id' was not returned with core information for "
                          f"show ID {show_id}")
    assert "description" in show, ("'description' was not returned with show "
                                   f"information for show ID {show_id}")


@pytest.mark.parametrize("show_ids", [[1082, 1162]])
def test_show_info_retrieve_core_info_by_ids(show_ids: List[int]):
    """Testing for :py:meth:`wwdtm.show.ShowInfoMultiple.retrieve_core_info_by_ids`

    :param show_id: Show ID to test retrieving show core information
    """
    info = ShowInfoMultiple(connect_dict=get_connect_dict())
    shows = info.retrieve_core_info_by_ids(show_ids)

    assert shows, "Core information all shows could not be retrieved"

    for show_id in show_ids:
        assert show_id in shows, ("Core information could not be retrieved for "
                                  f"show ID {show_id}")
        assert "id" in shows[show_id], ("'id' was not returned with core information "
                                        f"for show ID {show_id}")
        assert "description" in shows[show_id], ("'description' was not returned with "
                                                 f"show information for show ID {show_id}")


@pytest.mark.parametrize("show_id", [1082])
def test_show_info_retrieve_guest_info_all(show_id: int):
    """Testing for :py:meth:`wwdtm.show.ShowInfoMultiple.retrieve_guest_info_all`

    :param show_id: Show ID to test retrieving show guest information
        for all shows retrieved
    """
    info = ShowInfoMultiple(connect_dict=get_connect_dict())
    shows_guests = info.retrieve_guest_info_all()

    assert shows_guests, "Guest information all shows could not be retrieved"
    assert show_id in shows_guests, ("Guest information could not be retrieved for "
                                     f"show ID {show_id}")

    guests = shows_guests[show_id]
    if guests:
        assert "id" in guests[0], ("'id' was not returned for the first guest "
                                   f"for show ID {show_id}")
        assert "score" in guests[0], ("'score' was not returned for the first "
                                      f"guest for show ID {show_id}")


@pytest.mark.parametrize("show_ids", [[1082, 1162]])
def test_show_info_retrieve_guest_info_by_ids(show_ids: List[int]):
    """Testing for :py:meth:`wwdtm.show.ShowInfoMultiple.retrieve_guest_info_by_ids`

    :param show_ids: List of show IDs to test retrieving show guest
        information
    """
    info = ShowInfoMultiple(connect_dict=get_connect_dict())
    shows_guests = info.retrieve_guest_info_by_ids(show_ids)

    assert shows_guests, (f"Guest information for show IDs {show_ids} could not "
                          "be retrieved")

    for show_id in show_ids:
        assert show_id in shows_guests, (f"Guest information for show ID {show_id} "
                                         "could not be retrieved")

        guests = shows_guests[show_id]
        if guests:
            assert "id" in guests[0], ("'id' was not returned for the first guest "
                                       f"for show ID {show_id}")
            assert "score" in guests[0], ("'score' was not returned for the first "
                                          f"guest for show ID {show_id}")


@pytest.mark.parametrize("show_id", [1082])
def test_show_info_retrieve_panelist_info_all(show_id: int):
    """Testing for :py:meth:`wwdtm.show.ShowInfoMultiple.retrieve_panelist_info_all`

    :param show_id: Show ID to test retrieving show panelist information
        for all shows retrieved
    """
    info = ShowInfoMultiple(connect_dict=get_connect_dict())
    shows_panelists = info.retrieve_panelist_info_all()

    assert shows_panelists, f"Panelist information for all shows could not be retrieved"
    assert show_id in shows_panelists, ("Panelist information could not be retrieved "
                                        f"for show ID {show_id}")

    panelists = shows_panelists[show_id]
    assert "id" in panelists[0], (f"'id' was not returned for the first panelist "
                                  f"for show ID {show_id}")
    assert "score" in panelists[0], (f"'score' was not returned for the first "
                                     f"panelist for show ID {show_id}")


@pytest.mark.parametrize("show_ids", [[1082, 1162]])
def test_show_info_retrieve_panelist_info_by_ids(show_ids: List[int]):
    """Testing for :py:meth:`wwdtm.show.ShowInfoMultiple.retrieve_panelist_info_by_ids`

    :param show_ids: List of show IDs to test retrieving show panelist
        information
    """
    info = ShowInfoMultiple(connect_dict=get_connect_dict())
    shows_panelists = info.retrieve_panelist_info_by_ids(show_ids)

    assert shows_panelists, (f"Panelist information for show IDs {show_ids} could not "
                             "be retrieved")

    for show_id in shows_panelists:
        assert show_id in shows_panelists, ("Panelist information could not be "
                                            f"retrieved for show ID {show_id}")
        panelists = shows_panelists[show_id]
        if panelists:
            assert "id" in panelists[0], (f"'id' was not returned for the first panelist "
                                          f"for show ID {show_id}")
            assert "score" in panelists[0], (f"'score' was not returned for the first "
                                             f"panelist for show ID {show_id}")



# -*- coding: utf-8 -*-
# vim: set noai syntax=python ts=4 sw=4:
#
# Copyright (c) 2018-2021 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
"""Wait Wait Stats: Scorekeeper module"""
from wwdtm.scorekeeper.appearances import ScorekeeperAppearances
from wwdtm.scorekeeper.scorekeeper import Scorekeeper
from wwdtm.scorekeeper.utility import ScorekeeperUtility



# -*- coding: utf-8 -*-
# vim: set noai syntax=python ts=4 sw=4:
#
# Copyright (c) 2018-2021 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
"""Testing for object: :py:class:`wwdtm.guest.Guest`
"""
import json
from typing import Any, Dict

import pytest
from wwdtm.guest import Guest


@pytest.mark.skip
def get_connect_dict() -> Dict[str, Any]:
    """Read in database connection settings and return values as a
    dictionary.

    :return: A dictionary containing database connection settings
        for use by mysql.connector
    """
    with open("config.json", "r") as config_file:
        config_dict = json.load(config_file)
        if "database" in config_dict:
            return config_dict["database"]


def test_guest_retrieve_all():
    """Testing for :py:meth:`wwdtm.guest.Guest.retrieve_all`
    """
    guest = Guest(connect_dict=get_connect_dict())
    guests = guest.retrieve_all()

    assert guests, "No guests could be retrieved"
    assert "id" in guests[0], "'id' was not returned for the first list item"


def test_guest_retrieve_all_details():
    """Testing for :py:meth:`wwdtm.guest.Guest.retrieve_all_details`
    """
    guest = Guest(connect_dict=get_connect_dict())
    guests = guest.retrieve_all_details()

    assert guests, "No guests could be retrieved"
    assert "id" in guests[0], "'id' was not returned for first list item"
    assert "appearances" in guests[0], ("'appearances' was not returned for "
                                        "the first list item")


def test_guest_retrieve_all_ids():
    """Testing for :py:meth:`wwdtm.guest.Guest.retrieve_all_ids`
    """
    guest = Guest(connect_dict=get_connect_dict())
    ids = guest.retrieve_all_ids()

    assert ids, "No guest IDs could be retrieved"


def test_guest_retrieve_all_slugs():
    """Testing for :py:meth:`wwdtm.guest.Guest.retrieve_all_slugs`
    """
    guest = Guest(connect_dict=get_connect_dict())
    slugs = guest.retrieve_all_slugs()

    assert slugs, "No guest slug strings could be retrieved"


@pytest.mark.parametrize("guest_id", [976])
def test_guest_retrieve_by_id(guest_id: int):
    """Testing for :py:meth:`wwdtm.guest.Guest.retrieve_by_id`

    :param guest_id: Guest ID to test retrieving guest information
    """
    guest = Guest(connect_dict=get_connect_dict())
    info = guest.retrieve_by_id(guest_id)

    assert info, f"Guest ID {guest_id} not found"
    assert "name" in info, f"'name' was not returned for ID {guest_id}"


@pytest.mark.parametrize("guest_slug", ["tom-hanks"])
def test_guest_retrieve_by_slug(guest_slug: str):
    """Testing for :py:meth:`wwdtm.guest.Guest.retrieve_by_slug`

    :param guest_slug: Guest slug string to test retrieving guest
        information
    """
    guest = Guest(connect_dict=get_connect_dict())
    info = guest.retrieve_by_slug(guest_slug)

    assert info, f"Guest slug {guest_slug} not found"
    assert "name" in info, f"'name' was not returned for slug {guest_slug}"


@pytest.mark.parametrize("guest_id", [976])
def test_guest_retrieve_details_by_id(guest_id: int):
    """Testing for :py:meth:`wwdtm.guest.Guest.retrieve_details_by_id`

    :param guest_id: Guest ID to test retrieving guest details
    """
    guest = Guest(connect_dict=get_connect_dict())
    info = guest.retrieve_details_by_id(guest_id)

    assert info, f"Guest ID {guest_id} not found"
    assert "name" in info, f"'name' attribute was returned for ID {guest_id}"
    assert "appearances" in info, f"'appearances' was not returned for ID {guest_id}"


@pytest.mark.parametrize("guest_slug", ["tom-hanks"])
def test_guest_guest_retrieve_details_by_slug(guest_slug: str):
    """Testing for :py:meth:`wwdtm.guest.Guest.retrieve_details_by_slug`

    :param guest_slug: Guest slug string to test retrieving guest details
    """
    guest = Guest(connect_dict=get_connect_dict())
    info = guest.retrieve_details_by_slug(guest_slug)

    assert info, f"Guest slug {guest_slug} not found"
    assert "name" in info, f"'name' was not returned for slug {guest_slug}"
    assert "appearances" in info, f"'appearances' was not returned for slug {guest_slug}"



# -*- coding: utf-8 -*-
# vim: set noai syntax=python ts=4 sw=4:
#
# Copyright (c) 2018-2021 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
"""Package setup file"""

from setuptools import setup

setup(name="wwdtm",
      install_requires=[
          "mysql-connector-python==8.0.28",
          "numpy==1.22.1",
          "python-dateutil==2.8.2",
          "python-slugify==5.0.2",
          "pytz==2021.3",
      ]
      )



# -*- coding: utf-8 -*-
# vim: set noai syntax=python ts=4 sw=4:
#
# Copyright (c) 2018-2021 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
"""Wait Wait Don't Tell Me! Stats Panelist Statistics Retrieval Functions
"""
from functools import lru_cache
from typing import Any, Dict, Optional

from mysql.connector import connect
import numpy
from wwdtm.panelist.scores import PanelistScores
from wwdtm.panelist.utility import PanelistUtility
from wwdtm.validation import valid_int_id


class PanelistStatistics:
    """This class contains functions used to retrieve data from a copy
    of the Wait Wait Stats database and calculate statistics for
    panelists.

    :param connect_dict: Dictionary containing database connection
        settings as required by mysql.connector.connect
    :param database_connection: mysql.connector.connect database
        connection
    """

    def __init__(self,
                 connect_dict: Optional[Dict[str, Any]] = None,
                 database_connection: Optional[connect] = None):
        """Class initialization method.
        """
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

        self.scores = PanelistScores(database_connection=self.database_connection)
        self.utility = PanelistUtility(database_connection=self.database_connection)

    @lru_cache(typed=True)
    def retrieve_bluffs_by_id(self, panelist_id: int) -> Dict[str, int]:
        """Returns a dictionary containing the number of chosen Bluffs
        and correct Bluffs for the requested panelist ID.

        :param panelist_id: Panelist ID
        :return: Dictionary containing panelist Bluff counts. If
            panelist Bluff counts could not be returned, an empty
            dictionary will be returned.
        """
        cursor = self.database_connection.cursor(named_tuple=True)
        query = ("SELECT ( "
                 "SELECT COUNT(blm.chosenbluffpnlid) FROM ww_showbluffmap blm "
                 "JOIN ww_shows s ON s.showid = blm.showid "
                 "WHERE s.repeatshowid IS NULL AND blm.chosenbluffpnlid = %s "
                 ") AS chosen, ( "
                 "SELECT COUNT(blm.correctbluffpnlid) FROM ww_showbluffmap blm "
                 "JOIN ww_shows s ON s.showid = blm.showid "
                 "WHERE s.repeatshowid IS NULL AND blm.correctbluffpnlid = %s "
                 ") AS correct;")
        cursor.execute(query, (panelist_id, panelist_id, ))
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return {}

        return {
            "chosen": result.chosen,
            "correct": result.correct,
        }

    @lru_cache(typed=True)
    def retrieve_bluffs_by_slug(self, panelist_slug: str) -> Dict[str, int]:
        """Returns a dictionary containing the number of chosen Bluffs
        and correct Bluffs for the requested panelist slug string.

        :param panelist_slug: Panelist slug string
        :return: Dictionary containing panelist Bluff counts. If
            panelist Bluff counts could not be returned, an empty
            dictionary will be returned.
        """
        id_ = self.utility.convert_slug_to_id(panelist_slug)
        if not id_:
            return {}

        return self.retrieve_bluffs_by_id(id_)

    @lru_cache(typed=True)
    def retrieve_rank_info_by_id(self, panelist_id: int) -> Dict[str, int]:
        """Returns a dictionary with ranking information for the
        requested panelist ID.

        :param panelist_id: Panelist ID
        :return: Dictionary containing panelist ranking information. If
            panelist ranking information could not be returned, an empty
            dictionary will be returned.
        """
        if not valid_int_id(panelist_id):
            return {}

        cursor = self.database_connection.cursor(named_tuple=True)
        query = ("SELECT ( "
                 "SELECT COUNT(pm.showpnlrank) FROM ww_showpnlmap pm "
                 "JOIN ww_shows s ON s.showid = pm.showid "
                 "WHERE pm.panelistid = %s AND pm.showpnlrank = '1' AND "
                 "s.bestof = 0 and s.repeatshowid IS NULL) as 'first', ( "
                 "SELECT COUNT(pm.showpnlrank) FROM ww_showpnlmap pm "
                 "JOIN ww_shows s ON s.showid = pm.showid "
                 "WHERE pm.panelistid = %s AND pm.showpnlrank = '1t' AND "
                 "s.bestof = 0 and s.repeatshowid IS NULL) as 'first_tied', ( "
                 "SELECT COUNT(pm.showpnlrank) FROM ww_showpnlmap pm "
                 "JOIN ww_shows s ON s.showid = pm.showid "
                 "WHERE pm.panelistid = %s AND pm.showpnlrank = '2' AND "
                 "s.bestof = 0 and s.repeatshowid IS NULL) as 'second', ( "
                 "SELECT COUNT(pm.showpnlrank) FROM ww_showpnlmap pm "
                 "JOIN ww_shows s ON s.showid = pm.showid "
                 "WHERE pm.panelistid = %s AND pm.showpnlrank = '2t' AND "
                 "s.bestof = 0 and s.repeatshowid IS NULL) as 'second_tied', ( "
                 "SELECT COUNT(pm.showpnlrank) FROM ww_showpnlmap pm "
                 "JOIN ww_shows s ON s.showid = pm.showid "
                 "WHERE pm.panelistid = %s AND pm.showpnlrank = '3' AND "
                 "s.bestof = 0 and s.repeatshowid IS NULL "
                 ") as 'third';")
        cursor.execute(query, (panelist_id, panelist_id, panelist_id,
                               panelist_id, panelist_id, ))
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return {}

        return {
            "first": result.first,
            "first_tied": result.first_tied,
            "second": result.second,
            "second_tied": result.second_tied,
            "third": result.third,
        }

    @lru_cache(typed=True)
    def retrieve_rank_info_by_slug(self, panelist_slug: str) -> Dict[str, int]:
        """Returns a dictionary with ranking information for the
        requested panelist slug string.

        :param panelist_slug: Panelist slug string
        :return: Dictionary containing panelist ranking information. If
            panelist ranking information could not be returned, an empty
            dictionary will be returned.
        """
        id_ = self.utility.convert_slug_to_id(panelist_slug)
        if not id_:
            return {}

        return self.retrieve_rank_info_by_id(id_)

    @lru_cache(typed=True)
    def retrieve_statistics_by_id(self, panelist_id: int) -> Dict[str, Any]:
        """Returns a dictionary containing panelist statistics, ranking
        data, and scoring data for the requested panelist ID.

        :param panelist_id: Panelist ID
        :return: Dictionary containing panelist statistics. If panelist
            statistics could not be returned, an empty dictionary will
            be returned.
        """
        if not valid_int_id(panelist_id):
            return {}

        score_data = self.scores.retrieve_scores_by_id(panelist_id)
        ranks = self.retrieve_rank_info_by_id(panelist_id)

        if not score_data or not ranks:
            return {}

        appearance_count = len(score_data)
        scoring = {
            "minimum": int(numpy.amin(score_data)),
            "maximum": int(numpy.amax(score_data)),
            "mean": round(numpy.mean(score_data), 4),
            "median": int(numpy.median(score_data)),
            "standard_deviation": round(numpy.std(score_data), 4),
            "total": int(numpy.sum(score_data)),
        }

        ranks_first = round(100 * (ranks["first"] / appearance_count), 4)
        ranks_first_tied = round(100 * (ranks["first_tied"] / appearance_count), 4)
        ranks_second = round(100 * (ranks["second"] / appearance_count), 4)
        ranks_second_tied = round(100 * (ranks["second_tied"] / appearance_count), 4)
        ranks_third = round(100 * (ranks["third"] / appearance_count), 4)

        ranks_percentage = {
            "first": ranks_first,
            "first_tied": ranks_first_tied,
            "second": ranks_second,
            "second_tied": ranks_second_tied,
            "third": ranks_third,
        }

        ranking = {
            "rank": ranks,
            "percentage": ranks_percentage,
        }

        return {
            "scoring": scoring,
            "ranking": ranking,
        }

    @lru_cache(typed=True)
    def retrieve_statistics_by_slug(self, panelist_slug: str) -> Dict[str, Any]:
        """Returns a dictionary containing panelist statistics, ranking
        data, and scoring data for the requested panelist slug string.

        :param panelist_slug: Panelist slug string
        :return: Dictionary containing panelist statistics. If panelist
            statistics could not be returned, an empty dictionary will
            be returned.
        """
        id_ = self.utility.convert_slug_to_id(panelist_slug)
        if not id_:
            return {}

        return self.retrieve_statistics_by_id(id_)



# -*- coding: utf-8 -*-
# vim: set noai syntax=python ts=4 sw=4:
#
# Copyright (c) 2018-2021 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
"""Testing for object: :py:class:`wwdtm.scorekeeper.ScorekeeperUtility`
"""
import json
from typing import Any, Dict

import pytest
from wwdtm.scorekeeper import ScorekeeperUtility


@pytest.mark.skip
def get_connect_dict() -> Dict[str, Any]:
    """Read in database connection settings and return values as a
    dictionary.
    """
    with open("config.json", "r") as config_file:
        config_dict = json.load(config_file)
        if "database" in config_dict:
            return config_dict["database"]


@pytest.mark.parametrize("scorekeeper_id", [2])
def test_scorekeeper_utility_convert_id_to_slug(scorekeeper_id: int):
    """Testing for :py:meth:`wwdtm.scorekeeper.ScorekeeperUtility.convert_id_to_slug`

    :param scorekeeper_id: Scorekeeper ID to test converting into
        scorekeeper slug string
    """
    utility = ScorekeeperUtility(connect_dict=get_connect_dict())
    slug = utility.convert_id_to_slug(scorekeeper_id)

    assert slug, f"Scorekeeper slug for ID {scorekeeper_id} was not found"
    assert isinstance(slug, str), f"Invalid value returned for ID {scorekeeper_id}"


@pytest.mark.parametrize("scorekeeper_id", [-1])
def test_scorekeeper_utility_convert_invalid_id_to_slug(scorekeeper_id: int):
    """Negative testing for :py:meth:`wwdtm.scorekeeper.ScorekeeperUtility.convert_id_to_slug`

    :param scorekeeper_id: Scorekeeper ID to test failing to convert
        into scorekeeper slug string
    """
    utility = ScorekeeperUtility(connect_dict=get_connect_dict())
    slug = utility.convert_id_to_slug(scorekeeper_id)

    assert not slug, f"Scorekeeper slug for ID {scorekeeper_id} was found"


@pytest.mark.parametrize("scorekeeper_slug", ["korva-coleman"])
def test_scorekeeper_utility_convert_slug_to_id(scorekeeper_slug: str):
    """Testing for :py:meth:`wwdtm.scorekeeper.ScorekeeperUtility.convert_slug_to_id`

    :param scorekeeper_slug: Scorekeeper slug string to test converting
        into scorekeeper ID
    """
    utility = ScorekeeperUtility(connect_dict=get_connect_dict())
    id_ = utility.convert_slug_to_id(scorekeeper_slug)

    assert id_, f"Scorekeeper ID for slug {scorekeeper_slug} was found"
    assert isinstance(id_, int), f"Invalid value returned for slug {scorekeeper_slug}"


@pytest.mark.parametrize("scorekeeper_slug", ["corva-coleman"])
def test_scorekeeper_utility_convert_invalid_slug_to_id(scorekeeper_slug: str):
    """Negative testing for :py:meth:`wwdtm.scorekeeper.ScorekeeperUtility.convert_slug_to_id`

    :param scorekeeper_slug: Scorekeeper slug string to test failing to
        convert into scorekeeper ID
    """
    utility = ScorekeeperUtility(connect_dict=get_connect_dict())
    id_ = utility.convert_slug_to_id(scorekeeper_slug)

    assert not id_, f"Scorekeeper ID for slug {scorekeeper_slug} was not found"


@pytest.mark.parametrize("scorekeeper_id", [2])
def test_scorekeeper_utility_id_exists(scorekeeper_id: int):
    """Testing for :py:meth:`wwdtm.scorekeeper.ScorekeeperUtility.id_exists`

    :param scorekeeper_id: Scorekeeper ID to test if a scorekeeper
        exists
    """
    utility = ScorekeeperUtility(connect_dict=get_connect_dict())
    result = utility.id_exists(scorekeeper_id)

    assert result, f"Scorekeeper ID {scorekeeper_id} does not exist"


@pytest.mark.parametrize("scorekeeper_id", [-1])
def test_scorekeeper_utility_id_not_exists(scorekeeper_id: int):
    """Negative testing for :py:meth:`wwdtm.scorekeeper.ScorekeeperUtility.id_exists`

    :param scorekeeper_id: Scorekeeper ID to test if a scorekeeper does
        not exist
    """
    utility = ScorekeeperUtility(connect_dict=get_connect_dict())
    result = utility.id_exists(scorekeeper_id)

    assert not result, f"Scorekeeper ID {scorekeeper_id} exists"


@pytest.mark.parametrize("scorekeeper_slug", ["korva-coleman"])
def test_scorekeeper_utility_slug_exists(scorekeeper_slug: str):
    """Testing for :py:meth:`wwdtm.scorekeeper.ScorekeeperUtility.slug_exists`

    :param scorekeeper_slug: Scorekeeper slug string to test if a
        scorekeeper exists
    """
    utility = ScorekeeperUtility(connect_dict=get_connect_dict())
    result = utility.slug_exists(scorekeeper_slug)

    assert result, f"Scorekeeper slug {scorekeeper_slug} does not exist"


@pytest.mark.parametrize("scorekeeper_slug", ["corva-coleman"])
def test_scorekeeper_utility_slug_not_exists(scorekeeper_slug: str):
    """Negative testing for :py:meth:`wwdtm.scorekeeper.ScorekeeperUtility.slug_exists`

    :param scorekeeper_slug: Scorekeeper slug string to test if a
        scorekeeper does not exist
    """
    utility = ScorekeeperUtility(connect_dict=get_connect_dict())
    result = utility.slug_exists(scorekeeper_slug)

    assert not result, f"Scorekeeper slug {scorekeeper_slug} exists"



# -*- coding: utf-8 -*-
# vim: set noai syntax=python ts=4 sw=4:
#
# Copyright (c) 2018-2021 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
"""Testing for object: :py:class:`wwdtm.location.LocationRecordings`
"""
import json
from typing import Any, Dict

import pytest
from wwdtm.location import LocationRecordings


@pytest.mark.skip
def get_connect_dict() -> Dict[str, Any]:
    """Read in database connection settings and return values as a
    dictionary.

    :return: A dictionary containing database connection settings
        for use by mysql.connector
    """
    with open("config.json", "r") as config_file:
        config_dict = json.load(config_file)
        if "database" in config_dict:
            return config_dict["database"]


@pytest.mark.parametrize("location_id", [95])
def test_location_recordings_retrieve_recordings_by_id(location_id: int):
    """Testing for :py:meth:`wwdtm.location.LocationRecordings.retrieve_recordings_by_id`

    :param location_id: Location ID to test retrieving location
        recordings
    """
    recordings = LocationRecordings(connect_dict=get_connect_dict())
    recording = recordings.retrieve_recordings_by_id(location_id)

    assert "count" in recording, f"'count' was not returned for ID {location_id}"
    assert "shows" in recording, f"'shows' was not returned for ID {location_id}"


@pytest.mark.parametrize("location_slug", ["the-chicago-theatre-chicago-il"])
def test_location_recordings_retrieve_recordings_by_slug(location_slug: str):
    """Testing for :py:meth:`wwdtm.location.LocationRecordings.retrieve_recordings_by_slug`

    :param location_slug: Location slug string to test retrieving
        location recordings
    """
    recordings = LocationRecordings(connect_dict=get_connect_dict())
    recording = recordings.retrieve_recordings_by_slug(location_slug)

    assert "count" in recording, f"'count' was not returned for slug {location_slug}"
    assert "shows" in recording, f"'shows' was not returned for slug {location_slug}"
